import os.path
import sys


# Find data files when frozen. Adapted from cx_Freeze documentation:
# http://cx-freeze.readthedocs.org/en/latest/faq.html
def find_data_file(filename):
    if getattr(sys, 'frozen', False):
        datadir = os.path.dirname(sys.executable)
    else:
        datadir = os.path.dirname(__file__)

    return os.path.join(datadir, filename)



# VOD metadata file generator
# Copyright 2014 Bo Bayles (bbayles@gmail.com)
# See http://github.com/bbayles/vod_metadata for documentation and license

from vod_metadata.utils import find_data_file

config_path = find_data_file("vod_config.ini")
template_path = find_data_file("metadata_template.xml")

# Expose the main VodPackage class
from vod_metadata.vodpackage import VodPackage  # noqa
__all__ = ["VodPackage"]



from collections import namedtuple
import configparser

__all__ = ["ConfigurationError", "parse_config"]


class ConfigurationError(Exception):
    pass


VodConfig = namedtuple(
    "VodConfig",
    (
        "extensions,product,provider_id,prefix,title_category,provider,"
        "ecn_2009,mediainfo_path"
    )
)


def parse_config(config_path):
    config = configparser.ConfigParser()
    config.read(config_path)

    # extensions determines what files will be processed
    extensions = config.get(
        "Extensions", "extensions", fallback="mpg, ts, mp4"
    )
    extensions = set(".{}".format(x.strip()) for x in extensions.split(','))

    # mediainfo_path may or may not be set
    mediainfo_path = config.get("MediaInfo", "path", fallback=None)

    # Product can be up to 20 characters
    product = config.get("VOD", "product", fallback="MOD").strip()
    if len(product) > 20:
        raise ConfigurationError(
            "Configuration file error: product must be 20 characters or fewer"
        )

    # Provider_ID must be a lower-case domain name up to 20 characters
    provider_id = config.get("VOD", "provider_id", fallback="example.com")
    provider_id = provider_id.lower().strip()

    domain = provider_id.split(".")
    if (len(domain) != 2) or (len(domain[1]) == 1):
        raise ConfigurationError(
            "Configuration file error: provider_id must be a domain name."
        )
    if len(provider_id) > 20:
        raise ConfigurationError(
            "Configuration file error: provider_id must be less than 20 "
            "characters."
        )

    # Prefix must be 3 alphabetic characters
    prefix = config.get("VOD", "prefix", fallback="MSO").upper().strip()
    if (len(prefix) != 3) or (not prefix.isalpha()):
        raise ConfigurationError(
            "Configuration file error: prefix must be 3 alphabetic characters"
        )

    # Category must be a /-delimeted hierarchy of folder names, each folder
    # name 20 characters or fewer
    title_category = (
        config.get("VOD", "title_category", fallback="Testing/Videos").strip()
    )
    if any((len(folder) > 20 for folder in title_category.split("/"))):
        raise ConfigurationError(
            "Configuration file error: category title_category be a "
            "/-delimeted hierarchy of folder names, each folder name 20 "
            "characters or fewer"
        )

    # Provider can be up to 20 characters
    provider = config.get("VOD", "provider", fallback="001").strip()
    if len(provider) > 20:
        raise ConfigurationError(
            "Configuration file error: provider must be 20  characters or "
            "fewer"
        )

    # The ecn_2009 flag can be either "True" or "False"
    ecn_2009 = config.get("VOD", "ecn_2009", fallback="False")
    ecn_2009 = ecn_2009 == "True"

    return VodConfig(
        extensions,
        product,
        provider_id,
        prefix,
        title_category,
        provider,
        ecn_2009,
        mediainfo_path,
    )



from copy import deepcopy
from configparser import ConfigParser
from datetime import datetime
from io import BytesIO, open
import unittest
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch
import os.path

from vod_metadata import config_path, find_data_file
from vod_metadata.config_read import ConfigurationError, parse_config
from vod_metadata.md5_calc import md5_checksum
from vod_metadata.md_gen import generate_metadata
from vod_metadata.media_info import (
    call_MediaInfo,
    check_picture,
    check_video,
    find_MediaInfo,
    MediaInfoError,
)
from vod_metadata.vodpackage import VodPackage
from vod_metadata.xml_helper import etree, lxml, tobytes


@patch(
    "vod_metadata.config_read.configparser.ConfigParser.read",
    ConfigParser.read_file
)
class ConfigReadTests(unittest.TestCase):
    def setUp(self):
        with open(find_data_file(config_path), mode='r') as infile:
            self.config_lines = [line.strip() for line in infile if line]

    def _modify_key(self, key, value):
        target = "{} = ".format(key)
        commented_target = "# {} = ".format(key)
        if value is None:
            replace_str = ''
        else:
            replace_str = "{} = {}".format(key, value)

        ret = self.config_lines[:]
        for i, line in enumerate(ret):
            if line.startswith(target) or line.startswith(commented_target):
                ret[i] = replace_str
                return ret

    def test_extensions(self):
        # Test default value
        actual = parse_config(self._modify_key("extensions", None))[0]
        expected = {".mpg", ".ts", ".mp4"}
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("extensions", "mpg, mpg")
        actual = parse_config(config_lines)[0]
        expected = {'.mpg'}
        self.assertEqual(actual, expected)

    def test_product(self):
        # Test default value
        config_lines = self._modify_key("product", None)
        actual = parse_config(config_lines)[1]
        expected = "MOD"
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("product", "FOD")
        actual = parse_config(config_lines)[1]
        expected = "FOD"
        self.assertEqual(actual, expected)

        # Test incorrect value
        config_lines = self._modify_key("product", 'x' * 21)
        with self.assertRaises(ConfigurationError):
            parse_config(config_lines)[1]

    def test_provider_id(self):
        # Test default value
        config_lines = self._modify_key("provider_id", None)
        actual = parse_config(config_lines)[2]
        expected = "example.com"
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("provider_id", "EXAMPLE.org")
        actual = parse_config(config_lines)[2]
        expected = "example.org"
        self.assertEqual(actual, expected)

        # Test incorrect values
        incorrect_values = [
            "{}.com".format("x" * 17),  # Too long
            "examplecom",  # Too few dots
            "www.example.com",  # Too many dots
            "example.c",  # The bit after the dot is too short
        ]
        for value in incorrect_values:
            config_lines = self._modify_key("provider_id", value)
            with self.assertRaises(ConfigurationError):
                parse_config(config_lines)[2]

    def test_prefix(self):
        # Test default value
        config_lines = self._modify_key("prefix", None)
        actual = parse_config(config_lines)[3]
        expected = "MSO"
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("prefix", "ABC")
        actual = parse_config(config_lines)[3]
        expected = "ABC"
        self.assertEqual(actual, expected)

        # Test incorrect values
        incorrect_values = [
            "ABCD",  # Too many letters
            "!!!",  # Not alphanumeric
        ]
        for value in incorrect_values:
            config_lines = self._modify_key("prefix", value)
            with self.assertRaises(ConfigurationError):
                parse_config(config_lines)[3]

    def test_category(self):
        # Test default value
        config_lines = self._modify_key("title_category", None)
        actual = parse_config(config_lines)[4]
        expected = "Testing/Videos"
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("title_category", "Testing/HD")
        actual = parse_config(config_lines)[4]
        expected = "Testing/HD"
        self.assertEqual(actual, expected)

        # Test incorrect value
        config_lines = self._modify_key("title_category", 'x' * 21)
        with self.assertRaises(ConfigurationError):
            parse_config(config_lines)[4]

    def test_provider(self):
        # Test default value
        config_lines = self._modify_key("provider", None)
        actual = parse_config(config_lines)[5]
        expected = "001"
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("provider", "002")
        actual = parse_config(config_lines)[5]
        expected = "002"
        self.assertEqual(actual, expected)

        # Test incorrect value
        config_lines = self._modify_key("provider", 'x' * 21)
        with self.assertRaises(ConfigurationError):
            parse_config(config_lines)[5]

    def test_ecn_2009(self):
        # Test default value
        config_lines = self._modify_key("ecn_2009", None)
        actual = parse_config(config_lines)[6]
        expected = False
        self.assertEqual(actual, expected)

        # Test custom value
        config_lines = self._modify_key("ecn_2009", "True")
        actual = parse_config(config_lines)[6]
        expected = True
        self.assertEqual(actual, expected)

    def test_mediainfo_path(self):
        # Test default value
        config_lines = self._modify_key("path", None)
        actual = parse_config(config_lines)[7]
        self.assertIsNone(actual)

        # Test custom value
        config_lines = self._modify_key("path", "/usr/bin/mediainfo")
        actual = parse_config(config_lines)[7]
        expected = "/usr/bin/mediainfo"
        self.assertEqual(actual, expected)

    def test_no_config(self):
        actual = parse_config([])
        expected = (
            {".mpg", ".ts", ".mp4"},
            "MOD",
            "example.com",
            "MSO",
            "Testing/Videos",
            "001",
            False,
            None,
        )
        self.assertEqual(actual, expected)


class Md5CalcTests(unittest.TestCase):
    def test_checksum(self):
        test_value = md5_checksum(reference_mp4)
        known_value = "f5f66bd6e6b2ed02153d6fa94787626c"
        self.assertEqual(test_value, known_value)


class MdGenTests(unittest.TestCase):
    @patch('vod_metadata.md_gen.random', autospec=True)
    @patch('vod_metadata.md_gen.datetime.datetime', autospec=True)
    def setUp(self, mock_datetime, mock_random):
        mock_random.randint.return_value = 1020
        mock_datetime.today.return_value = datetime(1999, 9, 9, 1, 2)
        vod_config = parse_config(find_data_file(config_path))
        self.vod_package = generate_metadata(reference_mp4, vod_config)
        self.ams_expected = {
            "Provider":  "001",
            "Product": "MOD",
            "Version_Major": '1',
            "Version_Minor": '0',
            "Creation_Date": "1999-09-09",
            "Provider_ID": "example.com",
        }

    def test_package(self):
        # XML path
        actual = os.path.split(self.vod_package.xml_path)[1]
        expected = "reference_1020.xml"
        self.assertEqual(actual, expected)

        # Package AMS values
        actual = self.vod_package.D_ams["package"]
        expected = self.ams_expected.copy()
        package_expected = {
            "Asset_Name": "reference 1020 (package)",
            "Description": "reference 1020 (package asset)",
            "Asset_Class": "package",
            "Asset_ID": "MSOP1999090901021020",
        }
        expected.update(package_expected)
        self.assertEqual(actual, expected)

        # Package APP values
        actual = self.vod_package.D_app["package"]
        expected = {"Metadata_Spec_Version": "CableLabsVOD1.1"}
        self.assertEqual(actual, expected)

    def test_title(self):
        # Title AMS values
        actual = self.vod_package.D_ams["title"]
        expected = self.ams_expected.copy()
        title_expected = {
            "Asset_Name": "reference 1020 (title)",
            "Description": "reference 1020 (title asset)",
            "Asset_Class": "title",
            "Asset_ID": "MSOT1999090901021020",
        }
        expected.update(title_expected)
        self.assertEqual(actual, expected)

        # Title APP values
        actual = self.vod_package.D_app["title"]
        expected = {
            "Type": "title",
            "Title_Brief": "reference 1020",
            "Title": "reference 1020",
            "Summary_Short": "reference 1020",
            "Rating": ["NR"],
            "Closed_Captioning": 'N',
            "Year": "1999",
            "Category": ["MSO Lab"],
            "Genre": ["Other"],
            "Show_Type": "Other",
            "Billing_ID": "1020B",
            "Licensing_Window_Start": "1999-09-09",
            "Licensing_Window_End": "2002-06-04",
            "Preview_Period": "300",
            "Provider_QA_Contact": "N/A"
        }
        self.assertEqual(actual, expected)

    def test_movie(self):
        # Movie AMS values
        actual = self.vod_package.D_ams["movie"]
        expected = self.ams_expected.copy()
        movie_expected = {
            "Asset_Name": "reference 1020 (movie)",
            "Description": "reference 1020 (movie asset)",
            "Asset_Class": "movie",
            "Asset_ID": "MSOM1999090901021020",
        }
        expected.update(movie_expected)
        self.assertEqual(actual, expected)

        # Movie APP values
        actual = self.vod_package.D_app["movie"]
        expected = {
            'Audio_Type': 'Stereo',
            'Bit_Rate': '275',
            'Codec': 'AVC HP@L30',
            'Content_CheckSum': 'f5f66bd6e6b2ed02153d6fa94787626c',
            'Content_FileSize': '251404',
            'Frame_Rate': '25',
            'Resolution': '480p',
            'Type': 'movie',
        }
        self.assertEqual(actual, expected)

        # Movie Content values
        actual = self.vod_package.D_content["movie"]
        expected = reference_mp4
        self.assertEqual(actual, expected)


class MediaInfoTests(unittest.TestCase):
    def setUp(self):
        self.D_reference = {
            "General": {
                "Count of audio streams": '1',
                "Count of audio streams": '1',
                "File size": '251404',
                "Overall bit rate": '274758',
            },
            "Video": {
                "Format profile": 'High 4:4:4 Predictive@L3.0',
                "Commercial name": 'AVC',
                "Frame rate": '25.000',
                "Height": '480',
                "Scan type": 'Progressive',
            },
        }

    @patch('vod_metadata.media_info.os.path.isfile', autospec=True)
    def test_find_MediaInfo(self, mock_isfile):
        mock_isfile.return_value = True
        self.assertIsNotNone(find_MediaInfo())

        mock_isfile.return_value = False
        with self.assertRaises(RuntimeError):
            find_MediaInfo()

    def test_call_MediaInfo(self):
        D = call_MediaInfo(reference_mp4)
        for section in self.D_reference.keys():
            for key, expected in self.D_reference[section].items():
                actual = D[section][key]
                self.assertEqual(actual, expected)

    @patch('vod_metadata.media_info.call_MediaInfo', autospec=True)
    def test_check_video(self, mock_call_MediaInfo):
        # No modification -> should return normally
        mock_call_MediaInfo.return_value = self.D_reference
        self.assertEqual(check_video(reference_mp4), self.D_reference)

        # No General or Video section -> should fail
        for key in self.D_reference.keys():
            D = deepcopy(self.D_reference)
            del D[key]
            mock_call_MediaInfo.return_value = D
            with self.assertRaises(MediaInfoError):
                check_video(reference_mp4)

        # Missing keys -> should fail
        for section in ("General", "Video"):
            for key in self.D_reference[section]:
                D = deepcopy(self.D_reference)
                del D[section][key]
                mock_call_MediaInfo.return_value = D
                with self.assertRaises(MediaInfoError):
                    check_video(reference_mp4)

    @patch('vod_metadata.media_info.call_MediaInfo', autospec=True)
    def test_check_picture(self, mock_call_MediaInfo):
        # No modification -> should return normally
        D_image = {"Image": {"Width": "320", "Height": "240"}}
        mock_call_MediaInfo.return_value = D_image
        self.assertEqual(check_picture(None), D_image)

        # Missing keys -> should fail
        for D_image in (
            {}, {"Image": {"Width": "320"}}, {"Image": {"Height": "320"}}
        ):
            mock_call_MediaInfo.return_value = D_image
            with self.assertRaises(MediaInfoError):
                check_picture(None)


class XmlHelperTests(unittest.TestCase):
    def setUp(self):
        self.zero = etree.Element('zero')
        self.one = etree.SubElement(self.zero, 'one')
        self.two = etree.SubElement(self.one, 'two', attrib={'key': 'value'})
        self.expected_lines = [
            b'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n',
            b'<!DOCTYPE ADI SYSTEM "ADI.DTD">\n',
        ]

        if lxml:
            self.expected_lines.append(
                b'<zero>\n  <one>\n    <two key="value"/>\n  </one>\n</zero>\n'
            )
        else:
            self.expected_lines.append(
                b'<zero>\n'
                b'  <one>\n'
                b'    <two key="value" />\n'
                b'  </one>\n</zero>\n'
            )

    def test_tobytes(self):
        actual = tobytes(self.zero)
        expected = b''.join(self.expected_lines)
        self.assertEqual(actual, expected)


class VodMetadataTests(unittest.TestCase):
    def test_dictionaries(self):
        vod_package = VodPackage(reference_xml)
        # Package
        self.assertEqual(vod_package.D_ams["package"], ams_package)
        self.assertEqual(vod_package.D_app["package"], app_package)
        # Title
        self.assertEqual(vod_package.D_ams["title"], ams_title)
        self.assertEqual(vod_package.D_app["title"], app_title)
        # Movie
        self.assertEqual(vod_package.D_ams["movie"], ams_movie)
        self.assertEqual(vod_package.D_app["movie"], app_movie)
        # Preview
        self.assertEqual(vod_package.D_ams["preview"], ams_preview)
        self.assertEqual(vod_package.D_app["preview"], app_preview)
        # Poster
        self.assertEqual(vod_package.D_ams["poster"], ams_poster)
        self.assertEqual(vod_package.D_app["poster"], app_poster)

    def test_roundtrip(self):
        ref_package = VodPackage(reference_xml)
        file_out = BytesIO()
        file_out.write(ref_package.write_xml())
        file_out.seek(0)
        new_package = VodPackage(file_out)
        self.assertEqual(file_out.getvalue(), new_package.write_xml())


# Reference values
script_path = os.path.abspath(__file__)
script_dir = os.path.split(script_path)[0]
reference_xml = os.path.join(script_dir, "reference.xml")
reference_mp4 = os.path.join(script_dir, "reference.mp4")

ams_package = {
    'Asset_Class': 'package',
    'Asset_ID': 'TSTP2003010204050001',
    'Asset_Name': 'Metadata test (package)',
    'Creation_Date': '2003-01-02',
    'Description': 'Metadata test (package asset)',
    'Product': 'PRD',
    'Provider': '001',
    'Provider_ID': 'example.com',
    'Version_Major': '1',
    'Version_Minor': '0',
}
app_package = {'Metadata_Spec_Version': 'CableLabsVOD1.1'}

ams_title = {
    'Asset_Class': 'title',
    'Asset_ID': 'TSTT2003010204050001',
    'Asset_Name': 'Metadata test (title)',
    'Creation_Date': '2003-01-02',
    'Description': 'Metadata test (title asset)',
    'Product': 'PRD',
    'Provider': '001',
    'Provider_ID': 'example.com',
    'Version_Major': '1',
    'Version_Minor': '0',
}
app_title = {
    'Billing_ID': '0241B',
    'Category': ['Test Category', 'Test Category/Second Level'],
    'Closed_Captioning': 'N',
    'Display_Run_Time': '01:02',
    'Genre': ['Other'],
    'Licensing_Window_End': '2023-01-02T23:59:59',
    'Licensing_Window_Start': '2003-01-02',
    'Preview_Period': '241',
    'Provider_QA_Contact': 'username@example.com',
    'Rating': ['NR'],
    'Run_Time': '01:02:03',
    'Show_Type': 'Other',
    'Summary_Short': 'Test Summary_Short',
    'Title': 'Test Title',
    'Title_Brief': 'Test Title_Brief',
    'Type': 'title',
    'Year': '2003',
}

ams_movie = {
    'Asset_Class': 'movie',
    'Asset_ID': 'TSTM2003010204050001',
    'Asset_Name': 'Metadata test (movie)',
    'Creation_Date': '2003-01-02',
    'Description': 'Metadata test (movie asset)',
    'Product': 'PRD',
    'Provider': '001',
    'Provider_ID': 'example.com',
    'Version_Major': '1',
    'Version_Minor': '0',
}
app_movie = {
    'Audio_Type': 'Stereo',
    'Bit_Rate': '3750',
    'Codec': 'MPEG2',
    'Content_CheckSum': '2680090e51970e67b412af35201b9053',
    'Content_FileSize': '252706816',
    'Frame_Rate': '30',
    'Resolution': '480i',
    'Type': 'movie',
}

ams_preview = {
    'Asset_Class': 'preview',
    'Asset_ID': 'TSTR2003010204050001',
    'Asset_Name': 'Metadata test (preview)',
    'Creation_Date': '2003-01-02',
    'Description': 'Metadata test (preview asset)',
    'Product': 'PRD',
    'Provider': '001',
    'Provider_ID': 'example.com',
    'Version_Major': '1',
    'Version_Minor': '0',
}
app_preview = {
    'Audio_Type': 'Stereo',
    'Bit_Rate': '3750',
    'Codec': 'MPEG2',
    'Content_CheckSum': 'e0b1971fbe52920806190207a3036455',
    'Content_FileSize': '252706816',
    'Frame_Rate': '30',
    'Rating': ['NR'],
    'Resolution': '480i',
    'Run_Time': '00:04:56',
    'Type': 'preview',
}

ams_poster = {
    'Asset_Class': 'poster',
    'Asset_ID': 'TSTI2003010204050001',
    'Asset_Name': 'Metadata test (poster)',
    'Creation_Date': '2003-01-02',
    'Description': 'Metadata test (poster asset)',
    'Product': 'PRD',
    'Provider': '001',
    'Provider_ID': 'example.com',
    'Version_Major': '1',
    'Version_Minor': '0',
}
app_poster = {
    'Content_CheckSum': '410b51e0f900502809a22691537b67e3',
    'Content_FileSize': '252706816',
    'Image_Aspect_Ratio': '320x240',
    'Type': 'poster',
}

if __name__ == "__main__":
    unittest.main()



# VOD metadata file generator
# Copyright 2014 Bo Bayles (bbayles@gmail.com)
# See http://github.com/bbayles/vod_metadata for documentation and license
from __future__ import print_function

from argparse import ArgumentParser
from io import open
from os import chdir, getcwd, listdir
from os.path import splitext

from vod_metadata import config_path
from vod_metadata.config_read import parse_config
from vod_metadata.md_gen import generate_metadata


# Accept arguments from the command line
parser = ArgumentParser()
parser.add_argument(
    '--video-dir', help='Specify the path to a directory of videos to process'
)
parser.add_argument(
    '--mediainfo-path', help='Specify the path to the MediaInfo executable'
)
args = parser.parse_args()

if __name__ == "__main__":
    # Retrieve the user's configuration, overriding with command line arguments
    vod_config = parse_config(config_path)
    if args.mediainfo_path is not None:
        vod_config = vod_config._replace(mediainfo_path=args.mediainfo_path)

    # If a directory was specified, switch to it
    if args.video_dir is not None:
        chdir(args.video_dir)

    for file_path in listdir(getcwd()):
        # Only process movie files
        file_name, file_ext = splitext(file_path)
        if file_ext not in vod_config.extensions:
            continue

        # Create the VodPackage instace
        print("Processing {}...".format(file_path))
        vod_package = generate_metadata(file_path, vod_config)

        # Write the result
        s = vod_package.write_xml(rewrite=True)
        with open(vod_package.xml_path, "wb") as outfile:
            _ = outfile.write(s)



from __future__ import division
from io import open
import os.path

from vod_metadata import config_path
from vod_metadata.config_read import parse_config
from vod_metadata.md5_calc import md5_checksum
from vod_metadata.media_info import check_video, check_picture
from vod_metadata.xml_helper import etree, tobytes

__all__ = ["MissingElement", "InvalidMpeg", "VodPackage"]


class MissingElement(Exception):
    pass


class InvalidMpeg(Exception):
    pass


class VodPackage(object):

    # Some metadata attributes can appear more than once in the App_Data
    # sections.
    # TODO: Audio_Type is one of these, but this is not implemented yet
    _multiples = {
        "Provider_Content_Tier", "Subscriber_View_Limit", "Rating",
        "MSORating", "Advisories", "Audience", "Actors", "Director",
        "Producers", "Category", "Genre", "Chapter", "Recording_Artist",
        "Song_Title", "Languages", "Subtitle_Languages", "Dubbed_Languages"
    }

    def __init__(self, xml_path, vod_config=None):
        # Retrieve configuration if it's not set already
        if vod_config is None:
            self.vod_config = parse_config(config_path)
        else:
            self.vod_config = vod_config

        # The ECN 2009 options are not always supported - check configuration
        # to see whether they should be added.
        if self.vod_config.ecn_2009:
            self.param_skip = {"Resolution", "Frame_Rate", "Codec"}
        else:
            self.param_skip = set()

        self.xml_path = xml_path
        self.tree = etree.parse(self.xml_path)

        # The CableLabs VOD Metadata 1.1 specification stores metadata in "AMS"
        # and "App_Data" tags. The files that are part of the package are
        # stored in "Content" tags.
        self.D_ams = {}
        self.D_app = {}
        self.D_content = {}

        ADI = self.tree.getroot()
        self._read_package(ADI)
        self._read_title(ADI)
        self._read_elements(ADI)

        self.has_preview = "preview" in self.D_ams
        self.has_poster = "poster" in self.D_ams
        self.is_delete = self.D_ams.get("Verb", '') == "DELETE"
        self.is_update = self.D_ams["package"]["Version_Major"] != "1"

    def _read_package(self, ADI):
        package_Metadata = ADI.find("Metadata")
        self.D_ams["package"] = package_Metadata.find("AMS").attrib
        self.D_app["package"] = self._parse_App_Data(package_Metadata)

    def _read_title(self, ADI):
        title_Metadata = ADI.find("Asset").find("Metadata")
        self.D_ams["title"] = title_Metadata.find("AMS").attrib
        self.D_app["title"] = self._parse_App_Data(title_Metadata)

    def _read_elements(self, ADI):
        # Asset elements section: "movie", "poster", and "preview" are allowed.
        for ae_Asset in ADI.find("Asset").findall("Asset"):
            ae_Metadata = ae_Asset.find("Metadata")
            ae_AMS = ae_Metadata.find("AMS")
            ae_type = ae_AMS.attrib["Asset_Class"]
            self.D_ams[ae_type] = ae_AMS.attrib
            self.D_app[ae_type] = self._parse_App_Data(ae_Metadata)
            if ae_Asset.find("Content") is not None:
                self.D_content[ae_type] = (
                    ae_Asset.find("Content").attrib["Value"]
                )

    def _parse_App_Data(self, ae_Metadata):
        D = {}
        for App_Data in ae_Metadata.findall("App_Data"):
            key = App_Data.attrib["Name"]
            value = App_Data.attrib["Value"]
            # Some App_Data fields can occur more than once and are treated as
            # a list
            if key in self._multiples:
                if key in D:
                    D[key].append(value)
                else:
                    D[key] = [value]
            # Others can only occur once and are treated as plain values
            else:
                D[key] = value

        return D

    def _write_App_Data(self, ae_type, parent_Metadata):
        for key, value in sorted(
            self.D_app[ae_type].items(), key=lambda x: x[0]
        ):
            # Configuration controls whether certain values get skipped
            if key in self.param_skip:
                continue
            # Some of the App_Data tags can be repeated
            if key in self._multiples:
                for v in value:
                    ae_App_Data = etree.SubElement(parent_Metadata, "App_Data")
                    ae_App_Data.set("App", self.D_ams["package"]["Product"])
                    ae_App_Data.set("Name", key)
                    ae_App_Data.set("Value", v)
            # Others are only allowed to appear once
            else:
                ae_App_Data = etree.SubElement(parent_Metadata, "App_Data")
                ae_App_Data.set("App", self.D_ams["package"]["Product"])
                ae_App_Data.set("Name", key)
                ae_App_Data.set("Value", value)

    def write_xml(self, rewrite=False):
        # A movie element is required by this library
        if "movie" not in self.D_ams:
            raise MissingElement("Package does not specify a movie element")

        # Over-write the given XML values with the ones determined by scanning
        # the video if needed
        if rewrite:
            self.check_files()

        # Root element
        ADI = etree.Element("ADI")

        # Package asset
        package_Metadata = etree.SubElement(ADI, "Metadata")
        package_AMS = etree.SubElement(package_Metadata, "AMS")
        for key, value in sorted(
            self.D_ams["package"].items(), key=lambda x: x[0]
        ):
            package_AMS.set(key, value)
        self._write_App_Data("package", package_Metadata)

        # Title asset
        title_Asset = etree.SubElement(ADI, "Asset")
        title_Metadata = etree.SubElement(title_Asset, "Metadata")
        title_AMS = etree.SubElement(title_Metadata, "AMS")
        for key, value in sorted(
            self.D_ams["title"].items(), key=lambda x: x[0]
        ):
            title_AMS.set(key, value)
        self._write_App_Data("title", title_Metadata)

        # Asset elements
        for ae_type in ("movie", "preview", "poster"):
            if ae_type not in self.D_ams:
                continue
            ae_Asset = etree.SubElement(title_Asset, "Asset")
            ae_Metadata = etree.SubElement(ae_Asset, "Metadata")
            # AMS section
            ae_AMS = etree.SubElement(ae_Metadata, "AMS")
            for key, value in sorted(
                self.D_ams[ae_type].items(), key=lambda x: x[0]
            ):
                ae_AMS.set(key, value)
            self._write_App_Data(ae_type, ae_Metadata)
            # Content element, if it is present
            if ae_type in self.D_content:
                ae_Content = etree.SubElement(ae_Asset, "Content")
                ae_Content.set("Value", self.D_content[ae_type])

        return tobytes(ADI)

    def overwrite_xml(self, rewrite=False):
        s = self.write_xml(rewrite)
        with open(self.xml_path, mode="wb") as outfile:
            outfile.write(s)

    def files_present(self):
        # Check the referenced content files for existence. If they are all
        # present return True. Otherwise return False.
        for ae_type, ae_name in self.D_content.items():
            ae_dir = os.path.split(self.xml_path)[0]
            ae_path = os.path.join(ae_dir, ae_name)
            if not os.path.isfile(ae_path):
                return False

        return True

    def check_files(self):
        for ae_type, ae_name in self.D_content.items():
            # Check to make sure the referenced files exist in the same
            # directory as the XML file
            ae_dir = os.path.split(self.xml_path)[0]
            ae_path = os.path.join(ae_dir, ae_name)
            if not os.path.isfile(ae_path):
                msg = "Package's {} element is missing - {}"
                raise MissingElement(msg.format(ae_type, ae_path))
            # Set the file size and checksum values
            self.D_app[ae_type]["Content_FileSize"] = str(
                os.path.getsize(ae_path)
            )
            self.D_app[ae_type]["Content_CheckSum"] = md5_checksum(ae_path)
            # Use MediaInfo to determine the correct information about the
            # content files
            if ae_type == "poster":
                self._scan_image(ae_path)
            else:
                self._scan_video(ae_type, ae_path)

    def _remove_ae(self, ae_type):
        try:
            del self.D_ams[ae_type]
            del self.D_app[ae_type]
            del self.D_content[ae_type]
        except KeyError:
            msg = "Package does not content a {} element"
            raise MissingElement(msg.format(ae_type))

    def remove_preview(self):
        self._remove_ae("preview")
        self.has_preview = False

    def remove_poster(self):
        self._remove_ae("poster")
        self.has_poster = False

    def make_update(self):
        for ae_type in self.D_ams:
            new_version = int(self.D_ams[ae_type]["Version_Major"]) + 1
            self.D_ams[ae_type]["Version_Major"] = str(new_version)

        # The Content element shouldn't be present for updates, per section 8.1
        # of the ADI spec.
        self.D_content = {}
        self.is_update = True

    def make_delete(self):
        for ae_type in self.D_ams:
            self.D_ams[ae_type]["Verb"] = "DELETE"

        self.is_delete = True

    def _scan_video(self, ae_type, ae_path):
        mpeg_info = check_video(ae_path, self.vod_config.mediainfo_path)

        # Calculate the run time of the video
        duration_s = round(float(mpeg_info["General"]["Duration"]) / 1000)
        duration_h, duration_s = divmod(duration_s, 3600)
        duration_m, duration_s = divmod(duration_s, 60)
        duration_h = format(duration_h, "02")
        duration_m = format(duration_m, "02")
        duration_s = format(duration_s, "02")
        # For the movie asset, the Run_Time and Display_Run_Time are given as
        # part of the title's metadata. For the preview asset, the Run_Time is
        # given as part of the preview's metadata
        if ae_type == "movie":
            self.D_app["title"]["Run_Time"] = "{}:{}:{}".format(
                duration_h, duration_m, duration_s
            )
            self.D_app["title"]["Display_Run_Time"] = "{}:{}".format(
                duration_h, duration_m
            )
        elif ae_type == "preview":
            self.D_app["preview"]["Run_Time"] = "{}:{}:{}".format(
                duration_h, duration_m, duration_s
            )

        # Determine the movie's codec
        commercial_name = mpeg_info["Video"]["Commercial name"]
        format_profile = mpeg_info["Video"]["Format profile"]
        if commercial_name == "MPEG-2 Video":
            self.D_app[ae_type]["Codec"] = "MPEG2"
        elif commercial_name == "AVC":
            avc_profile = format_profile[0]
            avc_level = format_profile[format_profile.find("@"):].replace(
                ".", ""
            )
            self.D_app[ae_type]["Codec"] = "AVC {}P{}".format(
                avc_profile, avc_level
            )
        else:
            raise InvalidMpeg("Could not determine codec for {}".format(
                self.D_content[ae_type])
            )

        # Determine the audio type
        audio_type = int(mpeg_info["Audio"].get("Channel(s)", 0))
        self.D_app[ae_type]["Audio_Type"] = (
            "Stereo" if (audio_type > 1) else "Mono"
        )

        # Determine the geometry
        movie_resolution_height = mpeg_info["Video"]["Height"]
        move_resolution_scan = mpeg_info["Video"]["Scan type"][0].lower()
        self.D_app[ae_type]["Resolution"] = "{}{}".format(
            movie_resolution_height, move_resolution_scan
        )

        # Determine the movie's frame rate and bitrate (actually kilobit rate)
        frame_rate = float(mpeg_info["Video"]["Frame rate"])
        self.D_app[ae_type]["Frame_Rate"] = format(frame_rate, '.0f')

        bit_rate = float(mpeg_info["General"]["Overall bit rate"]) / 1000
        self.D_app[ae_type]["Bit_Rate"] = format(bit_rate, '.0f')

    def _scan_image(self, ae_path):
        img_info = check_picture(ae_path, self.vod_config.mediainfo_path)
        img_width = img_info["Image"]["Width"]
        img_height = img_info["Image"]["Height"]
        self.D_app["poster"]["Image_Aspect_Ratio"] = "{}x{}".format(
            img_width, img_height
        )



# lxml's etree.tostring() function supports a different set of arguments than
# Python's xml.etree.ElementTree.tostring() function. This module calls lxml's
# if it is installed, otherwise it uses Python's.
try:
    from lxml import etree
    lxml = True
except ImportError:
    import xml.etree.ElementTree as etree
    lxml = False


# Taken from Fredrik Lundh's effbot.org:
# http://effbot.org/zone/element-lib.htm#prettyprint
def indent(elem, level=0):
    i = "\n" + (level * "  ")
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level + 1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i


def tobytes(ADI):
    doctype = b'<!DOCTYPE ADI SYSTEM "ADI.DTD">'
    if lxml:
        return etree.tostring(
            ADI,
            xml_declaration=True,
            doctype=doctype,
            encoding='utf-8',
            pretty_print=True
        )
    else:
        declaration = b"<?xml version='1.0' encoding='utf-8'?>"
        indent(ADI)
        elements = etree.tostring(ADI, encoding="utf-8")

        return b'\n'.join((declaration, doctype, elements))



import datetime
import os
import random

from vod_metadata import template_path
from vod_metadata.vodpackage import VodPackage

__all__ = ["generate_metadata"]


def generate_metadata(file_path, vod_config):
    # Time-sensitive values
    timestamp = datetime.datetime.today()
    creation_date = timestamp.strftime("%Y-%m-%d")
    end_date = (timestamp + datetime.timedelta(days=999)).strftime("%Y-%m-%d")
    asset_id = timestamp.strftime("%Y%m%d%H%M")

    # Randomly-generated values
    suffix = format(random.randint(0, 9999), "04")
    title_billing_id = "{}B".format(suffix)

    # Start with a minimal metadata template
    vod_package = VodPackage(template_path, vod_config=vod_config)
    file_name = os.path.splitext(os.path.split(file_path)[1])[0]
    outfile_path = "{}_{}.xml".format(file_name, suffix)
    vod_package.xml_path = os.path.join(os.getcwd(), outfile_path)

    # Video file-specific value
    vod_package.D_content["movie"] = file_path
    vod_package.check_files()

    # Package section
    package_asset_name = "{} {} (package)".format(file_name[:20], suffix)
    package_description = "{} {} (package asset)".format(
        file_name[:20], suffix
    )
    package_asset_id = "{}P{}{}".format(vod_config.prefix, asset_id, suffix)

    vod_package.D_ams["package"] = {
        "Provider":  vod_config.provider,
        "Product": vod_config.product,
        "Asset_Name": package_asset_name,
        "Version_Major": '1',
        "Version_Minor": '0',
        "Description": package_description,
        "Creation_Date": creation_date,
        "Provider_ID": vod_config.provider_id,
        "Asset_ID": package_asset_id,
        "Asset_Class": "package"
    }
    vod_package.D_app["package"] = {"Metadata_Spec_Version": "CableLabsVOD1.1"}

    # Title section
    title_asset_name = "{} {} (title)".format(file_name[:20], suffix)
    title_description = "{} {} (title asset)".format(file_name[:20], suffix)
    title_asset_id = "{}T{}{}".format(vod_config.prefix, asset_id, suffix)
    title_title_brief = "{} {}".format(file_name[:14], suffix)
    title_title = "{} {}".format(file_name[:124], suffix)

    vod_package.D_ams["title"] = {
        "Provider":  vod_config.provider,
        "Product": vod_config.product,
        "Asset_Name": title_asset_name,
        "Version_Major": '1',
        "Version_Minor": '0',
        "Description": title_description,
        "Creation_Date": creation_date,
        "Provider_ID": vod_config.provider_id,
        "Asset_ID": title_asset_id,
        "Asset_Class": "title"
    }
    vod_package.D_app["title"] = {
        "Type": "title",
        "Title_Brief": title_title_brief,
        "Title": title_title,
        "Summary_Short": title_title,
        "Rating": ["NR"],
        "Closed_Captioning": 'N',
        "Year": timestamp.strftime("%Y"),
        "Category": [vod_config.title_category],
        "Genre": ["Other"],
        "Show_Type": "Other",
        "Billing_ID": title_billing_id,
        "Licensing_Window_Start": creation_date,
        "Licensing_Window_End": end_date,
        "Preview_Period": "300",
        "Provider_QA_Contact": "N/A"
    }

    # Movie section
    movie_asset_name = "{} {} (movie)".format(file_name[:20], suffix)
    movie_description = "{} {} (movie asset)".format(file_name[:20], suffix)
    movie_asset_id = "{}M{}{}".format(vod_config.prefix, asset_id, suffix)

    vod_package.D_ams["movie"] = {
        "Provider":  vod_config.provider,
        "Product": vod_config.product,
        "Asset_Name": movie_asset_name,
        "Version_Major": '1',
        "Version_Minor": '0',
        "Description": movie_description,
        "Creation_Date": creation_date,
        "Provider_ID": vod_config.provider_id,
        "Asset_ID": movie_asset_id,
        "Asset_Class": "movie"
    }
    vod_package.D_app["movie"]["Type"] = "movie"

    return vod_package



from io import open
import hashlib

__all__ = ["md5_checksum"]


def md5_checksum(file_path, chunk_bytes=4194304):
    """Return the MD5 checksum (hex digest) of the file"""

    with open(file_path, "rb") as infile:
        checksum = hashlib.md5()
        while 1:
            data = infile.read(chunk_bytes)
            if not data:
                break
            checksum.update(data)

    return checksum.hexdigest()



from setuptools import setup, find_packages
import sys

long_description = (
    "This project contains a library and tools for manipulating and "
    "generating metadata files that conform to the CableLabs VOD Metada 1.1 "
    "specification"
)

# Install requires configparser for Python 2.x
if sys.version_info[0] < 3:
    install_requires = ['configparser']
    tests_require = ['mock']
else:
    install_requires = []
    tests_require = []

setup(
    name='vod_metadata',
    version='2015.3.14',
    license='MIT',
    url='https://github.com/bbayles/vod_metadata',

    description='CableLabs VOD Metadata 1.1 library and tools',
    long_description="Library and tools for CableLabs VOD Metadata 1.1",

    author='Bo Bayles',
    author_email='bbayles@gmail.com',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Telecommunications Industry',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],
    keywords='cablelabs vod metadata',

    packages=find_packages(exclude=[]),
    test_suite='tests',

    # lxml is not required, but is recommended
    install_requires=install_requires,
    extras_require={'Speed':  ["lxml"]},
    tests_require=tests_require,

    package_data={'vod_metadata': ["*.ini", "*.mp4", "*.pth", "*.xml"]},
)



import collections
import os
import subprocess


class MediaInfoError(Exception):
    pass


def find_MediaInfo():
    typical_paths = [
        "C:\Program Files\MediaInfo\MediaInfo.exe",
        "C:\Program Files (x86)\MediaInfo\MediaInfo.exe",
        "/usr/bin/mediainfo",
    ]
    for mediainfo_path in typical_paths:
        if os.path.isfile(mediainfo_path):
            return mediainfo_path
    raise RuntimeError(
        "MediaInfo not found. Specify the path to MediaInfo in the "
        "configuration file."
    )


def call_MediaInfo(file_name, mediainfo_path=None):
    """Returns a dictionary of dictionaries with the output of
       MediaInfo -f file_name"""
    if mediainfo_path is None:
        mediainfo_path = find_MediaInfo()

    result = subprocess.check_output(
        [mediainfo_path, "-f", file_name], universal_newlines=True
    )
    D = collections.defaultdict(dict)
    for line in result.splitlines():
        line = line.split(':', 1)
        # Skip separators
        if line[0] == '':
            continue
        # Start a new section
        elif len(line) == 1:
            section = line[0].strip()
        # Record section key, value pairs
        else:
            k = line[0].strip()
            v = line[1].strip()
            if k not in D[section]:
                D[section][k] = v

    return D


def check_video(file_name, mediainfo_path=None):
    """
    Scans the given file with MediaInfo and returns the video and audio codec
    information if all the required parameters were found.
    """
    D = call_MediaInfo(file_name, mediainfo_path)
    err_msg = "Could not determine all video paramters"

    if ("General" not in D) or ("Video" not in D):
        raise MediaInfoError(err_msg)

    general_keys = ("Count of audio streams", "File size", "Overall bit rate")
    if any(k not in D["General"] for k in general_keys):
        raise MediaInfoError(err_msg)

    video_keys = (
        "Format profile",
        "Commercial name",
        "Frame rate",
        "Height",
        "Scan type",
    )
    if any(k not in D["Video"] for k in video_keys):
        raise MediaInfoError(err_msg)

    return D


def check_picture(file_name, mediainfo_path=None):
    """
    Scans the given file with MediaInfo and returns the picture
    information if all the required parameters were found.
    """
    D = call_MediaInfo(file_name, mediainfo_path)
    # Check that the file analyzed was a valid movie
    if (
        ("Image" not in D) or
        ("Width" not in D["Image"]) or
        ("Height" not in D["Image"])
    ):
        raise MediaInfoError("Could not determine all picture paramters")

    return D
